# Generated by Django 5.2 on 2024-08-09 08:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0034_alter_source_unique_together"),
    ]

    operations = [
        migrations.AlterField(
            model_name="protein",
            name="gene_symbol",
            field=models.CharField(
                db_index=True, default=None, max_length=256, null=True
            ),
        ),
    ]



import bionty.base as bt_base
import pandas as pd
import pytest
from bionty.base.entities._gene import MappingResult


@pytest.fixture(scope="module")
def genes():
    data = {
        "gene symbol": ["A1CF", "A1BG", "FANCD1", "corrupted"],
        "ncbi id": ["29974", "1", "5133", "corrupted"],
        "ensembl_gene_id": [
            "ENSG00000148584",
            "ENSG00000121410",
            "ENSG00000188389",
            "ENSG0000corrupted",
        ],
    }
    df = pd.DataFrame(data).set_index("ensembl_gene_id")

    gn = bt_base.Gene(source="ensembl")

    return df, gn


def test_gene_ensembl_inspect_hgnc_id(genes):
    df, gn = genes

    inspected_df = gn.inspect(df["ncbi id"], field=gn.ncbi_gene_id, return_df=True)

    inspect = inspected_df["__validated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, False])

    assert inspect.equals(expected_series)


def test_ensemblgene_download():
    from bionty.base.entities._gene import EnsemblGene

    ensembl_gene = EnsemblGene(organism="human", version="release-110")
    assert ensembl_gene._organism.name == "human"

    external_df = ensembl_gene.external_dbs()
    assert external_df.shape[0] > 1

    df = ensembl_gene.download_df(external_db_names={"HGNC": "hgnc_id"})
    assert df.shape[0] > 6000
    assert "hgnc_id" in df.columns


def test_ensemblgene_map_legacy_ids():
    gn = bt_base.Gene(organism="human", version="release-110")
    legacy_genes = [
        "ENSG00000280710",
        "ENSG00000261490",
        "ENSG00000203812",
        "ENSG00000204092",
        "ENSG00000215271",
    ]
    result = gn.map_legacy_ids(legacy_genes)
    assert result == MappingResult(
        mapped={
            "ENSG00000204092": "ENSG00000226070",
            "ENSG00000215271": "ENSG00000290292",
            "ENSG00000261490": "ENSG00000071127",
            "ENSG00000280710": "ENSG00000125304",
        },
        ambiguous={"ENSG00000203812": ["ENSG00000288859", "ENSG00000288825"]},
        unmapped=[],
    )

    result = gn.map_legacy_ids("ENSG00000280710")
    assert result == MappingResult(
        mapped={"ENSG00000280710": "ENSG00000125304"},
        ambiguous={},
        unmapped=[],
    )

    result = gn.map_legacy_ids(["ENSG00000280710"])
    assert result == MappingResult(
        mapped={"ENSG00000280710": "ENSG00000125304"},
        ambiguous={},
        unmapped=[],
    )


def test_old_ensembl_version():
    gene_ontology_102 = bt_base.Gene(
        source="ensembl", organism="mouse", version="release-102"
    )
    assert (
        "ENSMUSG00000021745"
        in gene_ontology_102.to_dataframe()["ensembl_gene_id"].values
    )

    gene_ontology_112 = bt_base.Gene(
        source="ensembl", organism="mouse", version="release-112"
    )
    assert (
        "ENSMUSG00000021745"
        not in gene_ontology_112.to_dataframe()["ensembl_gene_id"].values
    )



import shutil
from pathlib import Path

import requests  # type:ignore
import yaml  # type:ignore
from lamindb_setup.core.upath import UPath
from rich.progress import Progress

from bionty.base._settings import settings


def load_yaml(filename: str | Path):  # pragma: no cover
    with open(filename) as f:
        return yaml.safe_load(f)


def write_yaml(
    data: dict,
    filename: str | Path,
    sort_keys: bool = False,
    default_flow_style: bool = False,
):  # pragma: no cover
    with open(filename, "w") as f:
        yaml.dump(
            data,
            f,
            sort_keys=sort_keys,
            default_flow_style=default_flow_style,
        )


def url_download(
    url: str, localpath: str | Path | None = None, block_size: int = 1024, **kwargs
) -> str | Path | None:
    """Downloads a file to a specified path.

    Args:
        url: The URL to download.
        localpath: The path to download the file to.
        block_size: Buffer size in bytes for sending a file-like message body.
        **kwargs: Keyword arguments are passed to 'requests'

    Returns:
        The localpath file is downloaded to

    Raises:
        HttpError: If the request response is not 200 and OK.
    """
    if url.startswith("file://"):
        url = url.split("file://")[-1]
        shutil.copy(url, localpath)
        return localpath
    try:
        response = requests.get(url, stream=True, allow_redirects=True, **kwargs)
        response.raise_for_status()

        total_content_length = int(response.headers.get("content-length", 0))
        if localpath is None:
            localpath = url.split("/")[-1]

        if total_content_length > 5000000:
            with Progress(refresh_per_second=10, transient=True) as progress:
                task = progress.add_task(
                    "[red]downloading...", total=total_content_length
                )

                with open(localpath, "wb") as file:
                    for data in response.iter_content(block_size):
                        file.write(data)
                        progress.update(task, advance=block_size)
                # force the progress bar to 100% at the end
                progress.update(task, completed=total_content_length, refresh=True)
        else:
            with open(localpath, "wb") as file:
                for data in response.iter_content(block_size):
                    file.write(data)

        return localpath

    except requests.exceptions.HTTPError as err:
        if err.response.status_code == 404:
            raise requests.exceptions.HTTPError(
                f"URL not found (404): '{url}'. Check for typos."
            ) from err
        else:
            raise requests.exceptions.HTTPError(
                f"HTTP error ({err.response.status_code}): {url}."
            ) from err


def s3_bionty_assets(
    filename: str, localpath: Path = None, assets_base_url: str = "s3://bionty-assets"
):
    """Synchronizes a S3 file path with local file storage.

    If the file does not exist locally it gets downloaded to datasetdir/filename or the passed localpath.
    If the file does not exist on S3, the file does not get synchronized, no erroring.

    Args:
        filename: The suffix of the assets_base_url.
        localpath: Local base path of the file to sync.
        assets_base_url: The S3 base URL. Prefix of the filename.

    Returns:
        A Path object of the synchronized path.
    """
    if localpath is None:
        localpath = settings.datasetdir / filename
    else:  # it errors on reticulate if we pass a directory
        if localpath.exists():
            assert localpath.is_file(), (
                f"localpath {localpath} has to be a file path, not a directory"
            )
    # this requires s3fs, but it is installed by lamindb
    # skip_instance_cache=True to avoid interference with cached filesystems
    # especially with their dircache
    remote_path = (
        UPath(
            assets_base_url,
            skip_instance_cache=True,
            use_listings_cache=True,
            anon=True,
        )
        / filename
    )
    # check that the remote path exists and is available
    try:
        remote_stat = remote_path.stat()
    except (FileNotFoundError, PermissionError):
        return localpath
    # this is needed unfortunately because s3://bionty-assets doesn't have ListObjectsV2 for anonymous users.
    # Moreover, ListObjectsV2 is triggered inside .synchronize if no cache is present.
    # TODO: check if this is still needed
    parent_path = remote_path.parent.path.rstrip("/")
    remote_path.fs.dircache[parent_path] = [remote_stat.as_info()]
    # synchronize the remote path
    if hasattr(remote_path, "synchronize_to"):
        remote_path.synchronize_to(
            localpath, error_no_origin=False, print_progress=True
        )
    else:
        # UPath.synchronize is deprecated
        remote_path.synchronize(localpath, error_no_origin=False, print_progress=True)
    # clean the artificial cache
    del remote_path.fs.dircache[parent_path]

    return localpath



# Generated by Django 5.2 on 2025-02-16 21:50

import lamindb.base.fields
from django.db import migrations

import bionty.ids
from bionty._biorecord import encode_uid


def populate_uids(apps, schema_editor):
    Source = apps.get_model("bionty", "Source")
    for source in Source.objects.all():
        # Convert model instance to dictionary of fields
        kwargs = {
            field.name: getattr(source, field.name)
            for field in source._meta.fields
            if field.name != "uid"  # Exclude uid field itself
        }
        # Generate and save new uid
        source.uid = encode_uid(registry=Source, kwargs=kwargs)["uid"]
        source.save()


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0049_alter_schemacellmarker_cellmarker_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="source",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.source, max_length=8, unique=True
            ),
        ),
        migrations.RunPython(populate_uids, reverse_code=migrations.RunPython.noop),
    ]



doc_entites = """\
organism: `name` of `Organism` entity.
        source: The name of the source in the sources.yaml file.
                Get available sources with `.display_available_sources()`.
        version: The version of the ontology. Typically a date or an actual version.
                Get latest versions with `.display_available_sources()`.
"""
organism_removed_tmp = "\n".join(doc_entites.split("\n")[1:]).split("\n")
organism_removed_tmp[0] = organism_removed_tmp[0].removeprefix("        ")
organism_removed = "\n".join(organism_removed_tmp)


doc_curate = """\
df: DataFrame with a column of identifiers
        column: If `column` is `None`, checks the existing index for compliance with
                the default identifier.
                If `column` denotes an entity identifier, tries to map that identifier
                to the default identifier.
        field: The type of identifier for mapping.
"""



from __future__ import annotations

from typing import TYPE_CHECKING

from lamin_utils import logger

import bionty.base as bt_base

if TYPE_CHECKING:
    from pathlib import Path

    from lamindb.models import Artifact, SQLRecord


def sync_public_sources(update_currently_used: bool = False) -> None:
    """Add new public sources to the Source registry.

    This function registers new public sources in the Source registry.

    Public sources are in [sources.yaml](https://github.com/laminlabs/bionty/blob/main/bionty/base/sources.yaml).

    Args:
        update_currently_used: If True, set the latest version as currently_used.

    Example::

        from bionty.core import sync_public_sources

        sync_public_sources()
    """
    import bionty as bt
    from bionty._biorecord import list_biorecord_models

    records = bt.Source.objects.filter().all()
    df_sources = bt_base.display_sources().reset_index()
    bionty_models = list_biorecord_models(bt)
    for kwargs in df_sources.to_dict(orient="records"):
        if kwargs["entity"] in bionty_models:
            kwargs["entity"] = "bionty." + kwargs["entity"]
        record = records.filter(
            name=kwargs["name"],
            version=kwargs["version"],
            entity=kwargs["entity"],
            organism=kwargs["organism"],
        ).all()
        if len(record) == 0:
            record = bt.Source(**kwargs).save()
            logger.success(f"added new source: {record}")
        else:
            # update metadata fields
            record.update(**kwargs)
            # if the record is in the trash, restore it
            record.update(branch_id=1)
            logger.success(f"updated source: {record.one()}")

    if update_currently_used:
        logger.info("setting the latest version as currently_used...")
        # backwards compatible
        df = (
            records.to_dataframe() if hasattr(records, "to_dataframe") else records.df()
        )
        for (_, _, _), df_group in df.groupby(["entity", "organism", "name"]):
            if df_group.currently_used.sum() == 0:
                continue
            latest_uid = df_group.sort_values("version", ascending=False).uid.iloc[0]
            latest_record = records.get(uid=latest_uid)
            latest_record.currently_used = True
            latest_record.save()

    logger.success("synced up Source registry with the latest public sources")


def update_records_to_source(registry: type[SQLRecord], source: SQLRecord) -> None:
    """Update the existing records associated with old source to the new source.

    Args:
        registry: SQLRecord class to update.
        source: Source record to apply updates.
    """
    import lamindb as ln

    from bionty._source import filter_public_df_columns

    entity = registry.__get_name_with_module__()

    if entity != source.entity:
        raise ValueError(f"please pass a source record of the same entity: {entity}")

    # find records that need to be upgraded
    filter_kwargs = {
        "source__entity": entity,
        "source__name": source.name,
    }
    if hasattr(registry, "organism_id"):
        filter_kwargs["organism__name"] = source.organism
    records = registry.filter(**filter_kwargs).exclude(source=source).all()
    if len(records) == 0:
        return

    # determine the ontology ID field to use
    ontology_id_field = getattr(registry, "_ontology_id_field", "ontology_id")
    name_field = getattr(registry, "_name_field", "name")

    # get the new data from the source
    public_df = filter_public_df_columns(registry, registry.public(source=source))
    public_df.rename(columns={"parents": "_parents"}, inplace=True)
    if ontology_id_field not in public_df.columns:
        ontology_id_field = "stable_id" if "stable_id" in public_df.columns else None
        if not ontology_id_field:
            raise ValueError(
                f"'{ontology_id_field}' column is not found in the source dataframe."
            )

    # a dictionary of records indexed by ontology ID
    public_dict = {row[ontology_id_field]: row for _, row in public_df.iterrows()}

    # filter records that have matching ontology IDs
    records = records.filter(
        **{ontology_id_field + "__in": set(public_dict.keys())}
    ).all()

    # update records without artifacts
    # simply update the record if no artifacts are associated
    records_without_artifacts = records.filter(artifacts=None).all()
    records_to_update = []
    if records_without_artifacts:
        for record in records_without_artifacts:
            ontology_id = getattr(record, ontology_id_field)
            # update all fields from the new source
            for col in public_df.columns:
                setattr(record, col, public_dict[ontology_id].get(col))
            record.source_id = source.id
            records_to_update.append(record)

    # handle records with artifacts
    records_with_artifacts = records.exclude(artifacts=None).all()
    ontology_ids_for_new_records = []
    for record in records_with_artifacts:
        ontology_id = getattr(record, ontology_id_field)

        # if name changed, create a new record
        if (
            ontology_id in public_dict
            and hasattr(record, name_field)
            and getattr(record, name_field) != public_dict[ontology_id].get(name_field)
        ):
            ontology_ids_for_new_records.append(ontology_id)
        # otherwise, update the existing record
        else:
            for col in public_df.columns:
                setattr(record, col, public_dict[ontology_id].get(col))
            record.source_id = source.id
            records_to_update.append(record)
    # bulk update records with artifacts where name didn't change
    if records_to_update:
        logger.info(f"updating {len(records_to_update)} records...")
        ln.save(records_to_update)
        logger.success(f"{len(records_to_update)} records updated!")

    # create new records for those with name changes
    if ontology_ids_for_new_records:
        logger.info(f"creating {len(ontology_ids_for_new_records)} new records...")
        registry.from_values(
            ontology_ids_for_new_records,
            field=getattr(registry, ontology_id_field),
            source=source,
        ).save()
        logger.success(f"{len(ontology_ids_for_new_records)} new records created!")

    # set the source as currently used
    if not source.currently_used:
        source.currently_used = True
        source.save()


def register_source_in_bionty_assets(
    filepath: Path,
    source: SQLRecord,
    is_dataframe: bool = True,
    update: bool = False,
) -> Artifact:
    """Register a new source in the laminlabs/bionty-assets instance.

    Args:
        filepath: Path to the source file.
        source: Source record.
        is_dataframe: Whether the file is the DataFrame of the source.
        update: Whether to update the source if it already exists.

    Returns:
        Registered artifact record.

    Example::

        from bionty.core import register_source_in_bionty_assets

        source = Source.filter(entity="bionty.Gene", organism="human", name="ensembl", version="release-112").save()

        source_artifact = register_source_in_bionty_assets(
            "path/to/source.parquet",
            source,
            is_dataframe=True,
        )
    """
    import lamindb as ln

    # assert ln.setup.settings.instance.slug == "laminlabs/bionty-assets"

    if "." not in source.entity:
        raise ValueError(
            "source entity must be in form of 'module.ClassName', e.g. 'bionty.Gene'"
        )

    filepath = ln.UPath(filepath)
    if is_dataframe:
        assert filepath.suffix == ".parquet"

    artifact = ln.Artifact.filter(
        key=filepath.name, _key_is_virtual=False
    ).one_or_none()
    if artifact is not None:
        if not update:
            raise ValueError(
                f"artifact already exists: {artifact}\n   → pass `update=True` to update it"
            )
        else:
            artifact.replace(filepath)
    else:
        artifact = ln.Artifact(filepath, key=filepath.name)
        # NOTE: we use non-virtual keys for bionty-assets artifacts
        artifact._key_is_virtual = False
    artifact.save()

    if is_dataframe:
        organism, source_name, version, entity = filepath.stem.removeprefix(
            "df_"
        ).split("__")
        assert organism == source.organism
        assert source_name == source.name
        assert version == source.version
        assert entity == source.entity.split(".")[-1]
        source.dataframe_artifact = artifact
        source.save()
        logger.print(
            f"linked Source(uid={source.uid}) to dataframe_artifact {artifact}"
        )
    else:
        source.artifacts.add(artifact)
        logger.print(f"linked Source(uid={source.uid}) to {artifact}")

    return artifact



# Generated by Django 5.2 on 2024-09-09 07:52

import django.db.models.deletion
import lamindb.base.users
import lamindb.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0037_alter_cellline_source_alter_cellmarker_source_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="artifactcellline",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
    ]



import pytest
from bionty.base._ontology_url import (
    OntologyNotFoundError,
    OntologyVersionNotFoundError,
    get_ontology_url,
)


def test_get_ontology_url():
    # Test with a known prefix and version
    url, ver = get_ontology_url("OBA", "2022-05-11")
    assert url == "http://purl.obolibrary.org/obo/oba/releases/2022-05-11/oba.owl"
    assert ver == "2022-05-11"

    # Test with a known prefix and no version
    prefix = "OBA"
    url, ver = get_ontology_url(prefix)
    assert url is not None
    assert ver is not None

    # A wrong version
    with pytest.raises(OntologyVersionNotFoundError):
        get_ontology_url("OBA", "wrong_version")

    # Test with an unknown prefix
    with pytest.raises(OntologyNotFoundError):
        get_ontology_url("UNKNOWN_PREFIX")

    prefixes = [
        "ncbitaxon",
        "clo",
        "cl",
        "uberon",
        "mondo",
        "doid",
        "efo",
        "pato",
        "hp",
        "mp",
        "zp",
        "go",
        "pw",
        "dron",
        "hsapdv",
        "mmusdv",
        "hancestro",
    ]

    for prefix in prefixes:
        url, ver = get_ontology_url(prefix)
        assert url is not None
        assert ver is not None



import bionty.base as bt_base
import pytest


def test_unavailable_sources():
    with pytest.raises(ValueError):
        bt_base.CellType(source="random")


def test_diff_successful():
    disease_bt_1 = bt_base.Disease(source="mondo", version="2023-04-04")
    disease_bt_2 = bt_base.Disease(source="mondo", version="2023-02-06")

    new_entries, modified_entries = disease_bt_1.diff(disease_bt_2)
    assert len(new_entries) == 819
    assert len(modified_entries) == 249


def test_diff_value_errors():
    # Two different PublicOntology object types
    disease_bt = bt_base.Disease()
    phenotype_bt = bt_base.Phenotype()
    with pytest.raises(ValueError):
        disease_bt.diff(phenotype_bt)

    # Different sources
    disease_bt_1 = bt_base.Disease(source="mondo")
    disease_bt_2 = bt_base.Disease(source="doid")
    with pytest.raises(ValueError):
        disease_bt_1.diff(disease_bt_2)

    # Same version
    disease_bt_3 = bt_base.Disease(source="mondo", version="2023-04-04")
    disease_bt_4 = bt_base.Disease(source="mondo", version="2023-04-04")
    with pytest.raises(ValueError):
        disease_bt_3.diff(disease_bt_4)



from __future__ import annotations

from typing import Literal

from bionty.base._public_ontology import PublicOntology
from bionty.base.dev._doc_util import _doc_params

from ._shared_docstrings import doc_entites


@_doc_params(doc_entities=doc_entites)
class Phenotype(PublicOntology):
    """Phenotype.

    1. Human Phenotype Ontology
    https://hpo.jax.org/app/

    2. PATO - the Phenotype And Trait Ontology
    https://github.com/pato-ontology/pato

    3.Phecodes ICD10 map
    https://phewascatalog.org/phecodes_icd10

    3. PATO - Phenotype And Trait Ontology
    https://obofoundry.org/ontology/pato.html

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        organism: Literal["human", "all"] | None = None,
        source: Literal["hp", "phe", "pato"] | None = None,
        version: Literal[
            # HP
            "2025-05-06",
            "2024-04-26",
            "2024-03-06",
            "2023-06-17",
            "2023-04-05",
            "2023-01-27",
            # Pato
            "2025-05-14",
            "2024-03-28",
            "2023-05-18",
            # Phe
            "1.2",
        ]
        | None = None,
        **kwargs,
    ) -> None:
        super().__init__(
            source=source,
            version=version,
            organism=organism,
            include_id_prefixes={
                "hp": ["HP:"],
                "mp": ["MP:"],  # mp might require an exclusion prefix for mpath
                "zp": ["ZP:"],
                "pato": ["PATO:"],
            },
            **kwargs,
        )



# Generated by Django 5.2 on 2025-01-13 16:08

import django.db.models.deletion
import lamindb.base.fields
import lamindb.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0046_alter_cellline__aux_alter_cellmarker__aux_and_more"),
        ("lamindb", "0075_lamindbv1_part5"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="FeatureSetCellMarker",
            new_name="SchemaCellMarker",
        ),
        migrations.RenameModel(
            old_name="FeatureSetGene",
            new_name="SchemaGene",
        ),
        migrations.RenameModel(
            old_name="FeatureSetPathway",
            new_name="SchemaPathway",
        ),
        migrations.RenameModel(
            old_name="FeatureSetProtein",
            new_name="SchemaProtein",
        ),
        migrations.RenameField(
            model_name="gene",
            old_name="feature_sets",
            new_name="schemas",
        ),
        migrations.RenameField(
            model_name="protein",
            old_name="feature_sets",
            new_name="schemas",
        ),
        migrations.RenameField(
            model_name="cellmarker",
            old_name="feature_sets",
            new_name="schemas",
        ),
        migrations.RenameField(
            model_name="pathway",
            old_name="feature_sets",
            new_name="schemas",
        ),
        migrations.RenameField(
            model_name="schemacellmarker",
            old_name="featureset",
            new_name="schema",
        ),
        migrations.RenameField(
            model_name="schemagene",
            old_name="featureset",
            new_name="schema",
        ),
        migrations.RenameField(
            model_name="schemapathway",
            old_name="featureset",
            new_name="schema",
        ),
        migrations.RenameField(
            model_name="schemaprotein",
            old_name="featureset",
            new_name="schema",
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="schemas",
            field=models.ManyToManyField(
                related_name="cell_markers",
                through="bionty.SchemaCellMarker",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="schemas",
            field=models.ManyToManyField(
                related_name="genes", through="bionty.SchemaGene", to="lamindb.schema"
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="schemas",
            field=models.ManyToManyField(
                related_name="pathways",
                through="bionty.SchemaPathway",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="schemas",
            field=models.ManyToManyField(
                related_name="proteins",
                through="bionty.SchemaProtein",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="schemacellmarker",
            name="schema",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="schemagene",
            name="schema",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="schemapathway",
            name="schema",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="schemaprotein",
            name="schema",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="lamindb.schema",
            ),
        ),
    ]



# Generated by Django 5.2 on 2025-07-04 20:44

import django.db.models.deletion
import lamindb.base.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0054_alter_cellline_branch_alter_cellline_space_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="recordcellline",
            old_name="cellline",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordcellmarker",
            old_name="cellmarker",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordcelltype",
            old_name="celltype",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recorddevelopmentalstage",
            old_name="developmentalstage",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recorddisease",
            old_name="disease",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordethnicity",
            old_name="ethnicity",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordexperimentalfactor",
            old_name="experimentalfactor",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordgene",
            old_name="gene",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordorganism",
            old_name="organism",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordpathway",
            old_name="pathway",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordphenotype",
            old_name="phenotype",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordprotein",
            old_name="protein",
            new_name="value",
        ),
        migrations.RenameField(
            model_name="recordtissue",
            old_name="tissue",
            new_name="value",
        ),
        migrations.AlterField(
            model_name="recordcellline",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_cell_line",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordcellmarker",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_cell_marker",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordcelltype",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_cell_type",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recorddevelopmentalstage",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_developmental_stage",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recorddisease",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_disease",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordethnicity",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_ethnicity",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordexperimentalfactor",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_experimental_factor",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordgene",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_gene",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordorganism",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_organism",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordpathway",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_pathway",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordphenotype",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_phenotype",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordprotein",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="values_protein",
                to="lamindb.record",
            ),
        ),
        migrations.AlterField(
            model_name="recordtissue",
            name="record",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="value_tissue",
                to="lamindb.record",
            ),
        ),
    ]



# Generated by Django 5.1.12 on 2025-10-01 14:39

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0060_alter_cellline_is_locked_alter_cellmarker_is_locked_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="cellline",
            name="page",
        ),
        migrations.RemoveField(
            model_name="cellmarker",
            name="page",
        ),
        migrations.RemoveField(
            model_name="celltype",
            name="page",
        ),
        migrations.RemoveField(
            model_name="developmentalstage",
            name="page",
        ),
        migrations.RemoveField(
            model_name="disease",
            name="page",
        ),
        migrations.RemoveField(
            model_name="ethnicity",
            name="page",
        ),
        migrations.RemoveField(
            model_name="experimentalfactor",
            name="page",
        ),
        migrations.RemoveField(
            model_name="gene",
            name="page",
        ),
        migrations.RemoveField(
            model_name="organism",
            name="page",
        ),
        migrations.RemoveField(
            model_name="pathway",
            name="page",
        ),
        migrations.RemoveField(
            model_name="phenotype",
            name="page",
        ),
        migrations.RemoveField(
            model_name="protein",
            name="page",
        ),
        migrations.RemoveField(
            model_name="source",
            name="page",
        ),
        migrations.RemoveField(
            model_name="tissue",
            name="page",
        ),
    ]



import re

import pandas as pd
from django.core.exceptions import FieldDoesNotExist, ObjectDoesNotExist
from lamin_utils import logger

from .models import BioRecord, Organism


class OrganismNotSet(SystemExit):
    """The `organism` parameter was not passed or is not globally set."""

    pass


def create_or_get_organism_record(
    organism: str | Organism | None, registry: type[BioRecord], field: str | None = None
) -> Organism | None:
    """Create or get an organism record from the given organism name."""
    # return None if a registry doesn't require organism field
    organism_record = None
    if is_organism_required(registry):
        # using global setting of organism
        from .core._settings import settings
        from .models import Organism

        if organism is None and settings.organism is not None:
            logger.debug(f"using default organism = {settings.organism.name}")
            return settings.organism

        if isinstance(organism, Organism):
            organism_record = organism
        elif isinstance(organism, str):
            try:
                # existing organism record
                organism_record = Organism.objects.get(name=organism)
            except ObjectDoesNotExist:
                try:
                    # create a organism record from bionty reference
                    organisms = Organism.from_values([organism])
                    if len(organisms) == 0:
                        raise ValueError(
                            f"Organism {organism} can't be created from the bionty reference, check your spelling or create it manually."
                        )
                    organism_record = organisms[0].save()  # type:ignore
                except KeyError:
                    # no such organism is found in bionty reference
                    organism_record = None

        if organism_record is None:
            if hasattr(registry, "_ontology_id_field") and field in {
                registry._ontology_id_field,
                "uid",
            }:
                return None
            raise OrganismNotSet(
                f"{registry.__name__} requires to specify a organism name via `organism=` or `bionty.settings.organism=`!"
            )

    return organism_record


def is_organism_required(registry: type[BioRecord]) -> bool:
    """Check if the registry has an organism field and is required.

    Returns:
        True if the registry has an organism field and is required, False otherwise.
    """
    try:
        organism_field = registry._meta.get_field("organism")
        # organism is not required or not a relation
        if organism_field.null or not organism_field.is_relation:
            return False
        else:
            return True
    except FieldDoesNotExist:
        return False


def organism_from_ensembl_id(id: str, using_key: str | None) -> Organism | None:
    """Get organism record from ensembl id."""
    import bionty as bt
    from bionty.base.dev._io import s3_bionty_assets

    # below has to consume a file path and NOT a directory because otherwise it fails on reticulate
    localpath = s3_bionty_assets(
        ".lamindb/0QeqXlKq9aqW8aqe0000.parquet",
        bt.base.settings.dynamicdir / "ensembl_prefix.parquet",
    )
    ensembl_prefixes = pd.read_parquet(localpath).set_index("gene_prefix")

    prefix = (
        re.search(r"^[A-Za-z]+", id).group(0) if re.search(r"^[A-Za-z]+", id) else id
    )
    if prefix in ensembl_prefixes.index:
        organism_name = ensembl_prefixes.loc[prefix, "name"].lower()

        using_key = None if using_key == "default" else using_key

        organism_record = (
            bt.Organism.using(using_key).filter(name=organism_name).one_or_none()
        )
        if organism_record is None:
            organisms = bt.Organism.from_values([organism_name])
            if len(organisms) > 0:
                organism_record = organisms[0]
                organism_record.save(using=using_key)
            else:
                raise OrganismNotSet(
                    f"Organism {organism_name} can't be created from the source, check your spelling or create it manually."
                )
        return organism_record
    return None



# Generated by Django 5.0.6 on 2024-05-19 12:01

import django.db.models.deletion
import lamindb.base.users
import lamindb.models
from django.db import migrations, models

import bionty.ids


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("lamindb", "0070_lamindbv1_migrate_data"),
    ]

    operations = [
        migrations.CreateModel(
            name="ArtifactCellLine",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("cell_line_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="cell_line_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactcellline_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactCellMarker",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "cell_marker_ref_is_name",
                    models.BooleanField(default=None, null=True),
                ),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="cell_marker_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactcellmarker_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactCellType",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("cell_type_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="cell_type_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactcelltype_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactDevelopmentalStage",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "developmental_stage_ref_is_name",
                    models.BooleanField(default=None, null=True),
                ),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="developmental_stage_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactdevelopmentalstage_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactDisease",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("disease_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="disease_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactdisease_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactEthnicity",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("ethnicity_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ethnicity_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactethnicity_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactExperimentalFactor",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "experimental_factor_ref_is_name",
                    models.BooleanField(default=None, null=True),
                ),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="experimental_factor_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactexperimentalfactor_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactGene",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("gene_ref_is_symbol", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_symbol", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="gene_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactgene_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactOrganism",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("organism_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="organism_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactorganism_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactPathway",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("pathway_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="pathway_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactpathway_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactPhenotype",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("phenotype_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="phenotype_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactphenotype_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactProtein",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("protein_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="protein_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifactprotein_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactTissue",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("tissue_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="tissue_links",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="artifacttissue_links",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.IsLink, models.Model),
        ),
        migrations.CreateModel(
            name="CellLine",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="cell_lines",
                        through="bionty.ArtifactCellLine",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.cellline"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifactcellline",
            name="cellline",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.cellline",
            ),
        ),
        migrations.CreateModel(
            name="CellMarker",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.cellmarker,
                        max_length=12,
                        unique=True,
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True, default=None, max_length=64, unique=True
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                (
                    "gene_symbol",
                    models.CharField(
                        db_index=True, default=None, max_length=64, null=True
                    ),
                ),
                (
                    "ncbi_gene_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "uniprotkb_id",
                    models.CharField(
                        db_index=True, default=None, max_length=10, null=True
                    ),
                ),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="cell_markers",
                        through="bionty.ArtifactCellMarker",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifactcellmarker",
            name="cellmarker",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.cellmarker",
            ),
        ),
        migrations.CreateModel(
            name="CellType",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="cell_types",
                        through="bionty.ArtifactCellType",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.celltype"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifactcelltype",
            name="celltype",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.celltype",
            ),
        ),
        migrations.CreateModel(
            name="DevelopmentalStage",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="developmental_stages",
                        through="bionty.ArtifactDevelopmentalStage",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.developmentalstage"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifactdevelopmentalstage",
            name="developmentalstage",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.developmentalstage",
            ),
        ),
        migrations.CreateModel(
            name="Disease",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="diseases",
                        through="bionty.ArtifactDisease",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.disease"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifactdisease",
            name="disease",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.disease",
            ),
        ),
        migrations.CreateModel(
            name="Ethnicity",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="ethnicities",
                        through="bionty.ArtifactEthnicity",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.ethnicity"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifactethnicity",
            name="ethnicity",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.ethnicity",
            ),
        ),
        migrations.CreateModel(
            name="ExperimentalFactor",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                ("molecule", models.TextField(db_index=True, default=None, null=True)),
                (
                    "instrument",
                    models.TextField(db_index=True, default=None, null=True),
                ),
                (
                    "measurement",
                    models.TextField(db_index=True, default=None, null=True),
                ),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="experimental_factors",
                        through="bionty.ArtifactExperimentalFactor",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.experimentalfactor"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifactexperimentalfactor",
            name="experimental_factor",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.experimentalfactor",
            ),
        ),
        migrations.CreateModel(
            name="FeatureSetCellMarker",
            fields=[
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "cellmarker",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="bionty.cellmarker",
                    ),
                ),
                (
                    "featureset",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="lamindb.featureset",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.IsLink),
        ),
        migrations.AddField(
            model_name="cellmarker",
            name="feature_sets",
            field=models.ManyToManyField(
                related_name="cell_markers",
                through="bionty.FeatureSetCellMarker",
                to="lamindb.featureset",
            ),
        ),
        migrations.CreateModel(
            name="FeatureSetGene",
            fields=[
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "featureset",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="lamindb.featureset",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.IsLink),
        ),
        migrations.CreateModel(
            name="FeatureSetPathway",
            fields=[
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "featureset",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="lamindb.featureset",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.IsLink),
        ),
        migrations.CreateModel(
            name="FeatureSetProtein",
            fields=[
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "featureset",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="lamindb.featureset",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.IsLink),
        ),
        migrations.CreateModel(
            name="Gene",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.gene, max_length=12, unique=True
                    ),
                ),
                (
                    "symbol",
                    models.CharField(
                        db_index=True, default=None, max_length=64, null=True
                    ),
                ),
                (
                    "stable_id",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=64,
                        null=True,
                        unique=True,
                    ),
                ),
                (
                    "ensembl_gene_id",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=64,
                        null=True,
                        unique=True,
                    ),
                ),
                ("ncbi_gene_ids", models.TextField(default=None, null=True)),
                (
                    "biotype",
                    models.CharField(
                        db_index=True, default=None, max_length=64, null=True
                    ),
                ),
                ("description", models.TextField(default=None, null=True)),
                ("synonyms", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="genes",
                        through="bionty.ArtifactGene",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature_sets",
                    models.ManyToManyField(
                        related_name="genes",
                        through="bionty.FeatureSetGene",
                        to="lamindb.featureset",
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="featuresetgene",
            name="gene",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.gene",
            ),
        ),
        migrations.AddField(
            model_name="artifactgene",
            name="gene",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.gene",
            ),
        ),
        migrations.CreateModel(
            name="Organism",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True, default=None, max_length=64, unique=True
                    ),
                ),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                (
                    "scientific_name",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=64,
                        null=True,
                        unique=True,
                    ),
                ),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="organisms",
                        through="bionty.ArtifactOrganism",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.organism"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="gene",
            name="organism",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="genes",
                to="bionty.organism",
            ),
        ),
        migrations.AddField(
            model_name="cellmarker",
            name="organism",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="cell_markers",
                to="bionty.organism",
            ),
        ),
        migrations.AddField(
            model_name="artifactorganism",
            name="organism",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.organism",
            ),
        ),
        migrations.CreateModel(
            name="Pathway",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="pathways",
                        through="bionty.ArtifactPathway",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature_sets",
                    models.ManyToManyField(
                        related_name="pathways",
                        through="bionty.FeatureSetPathway",
                        to="lamindb.featureset",
                    ),
                ),
                (
                    "genes",
                    models.ManyToManyField(related_name="pathways", to="bionty.gene"),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.pathway"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="featuresetpathway",
            name="pathway",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.pathway",
            ),
        ),
        migrations.AddField(
            model_name="artifactpathway",
            name="pathway",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.pathway",
            ),
        ),
        migrations.CreateModel(
            name="Phenotype",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="phenotypes",
                        through="bionty.ArtifactPhenotype",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="bionty.phenotype"
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifactphenotype",
            name="phenotype",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.phenotype",
            ),
        ),
        migrations.CreateModel(
            name="Protein",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.protein, max_length=12, unique=True
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True, default=None, max_length=64, null=True
                    ),
                ),
                (
                    "uniprotkb_id",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=10,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("length", models.BigIntegerField(db_index=True, null=True)),
                (
                    "gene_symbol",
                    models.CharField(
                        db_index=True, default=None, max_length=64, null=True
                    ),
                ),
                ("ensembl_gene_ids", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="proteins",
                        through="bionty.ArtifactProtein",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature_sets",
                    models.ManyToManyField(
                        related_name="proteins",
                        through="bionty.FeatureSetProtein",
                        to="lamindb.featureset",
                    ),
                ),
                (
                    "organism",
                    models.ForeignKey(
                        default=None,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="proteins",
                        to="bionty.organism",
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="featuresetprotein",
            name="protein",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.protein",
            ),
        ),
        migrations.AddField(
            model_name="artifactprotein",
            name="protein",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.protein",
            ),
        ),
        migrations.CreateModel(
            name="PublicSource",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.source,
                        max_length=8,
                        unique=True,
                    ),
                ),
                ("entity", models.CharField(db_index=True, max_length=64)),
                ("organism", models.CharField(db_index=True, max_length=64)),
                ("currently_used", models.BooleanField(db_index=True, default=False)),
                ("source", models.CharField(db_index=True, max_length=64)),
                ("source_name", models.TextField(blank=True, db_index=True)),
                ("version", models.CharField(db_index=True, max_length=64)),
                ("url", models.TextField(default=None, null=True)),
                ("md5", models.TextField(default=None, null=True)),
                ("source_website", models.TextField(default=None, null=True)),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "unique_together": {("entity", "source", "organism", "version")},
            },
        ),
        migrations.AddField(
            model_name="protein",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="proteins",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="phenotype",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="phenotypes",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="pathway",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="pathways",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="organism",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="organisms",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="gene",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="genes",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="experimentalfactor",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="experimental_factors",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="ethnicity",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="ethnicities",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="disease",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="diseases",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="developmentalstage",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="developmental_stages",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="celltype",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="cell_types",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="cellmarker",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="cell_markers",
                to="bionty.publicsource",
            ),
        ),
        migrations.AddField(
            model_name="cellline",
            name="public_source",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="cell_lines",
                to="bionty.publicsource",
            ),
        ),
        migrations.CreateModel(
            name="Tissue",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="tissues",
                        through="bionty.ArtifactTissue",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(related_name="children", to="bionty.tissue"),
                ),
                ("previous_runs", models.ManyToManyField(to="lamindb.run")),
                (
                    "public_source",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="tissues",
                        to="bionty.publicsource",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "unique_together": {("name", "ontology_id")},
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.AddField(
            model_name="artifacttissue",
            name="tissue",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="artifact_links",
                to="bionty.tissue",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="phenotype",
            unique_together={("name", "ontology_id")},
        ),
        migrations.AlterUniqueTogether(
            name="pathway",
            unique_together={("name", "ontology_id")},
        ),
        migrations.AlterUniqueTogether(
            name="experimentalfactor",
            unique_together={("name", "ontology_id")},
        ),
        migrations.AlterUniqueTogether(
            name="ethnicity",
            unique_together={("name", "ontology_id")},
        ),
        migrations.AlterUniqueTogether(
            name="disease",
            unique_together={("name", "ontology_id")},
        ),
        migrations.AlterUniqueTogether(
            name="developmentalstage",
            unique_together={("name", "ontology_id")},
        ),
        migrations.AlterUniqueTogether(
            name="celltype",
            unique_together={("name", "ontology_id")},
        ),
        migrations.AlterUniqueTogether(
            name="cellline",
            unique_together={("name", "ontology_id")},
        ),
    ]



from __future__ import annotations

from typing import Literal

from bionty.base._public_ontology import PublicOntology
from bionty.base.dev._doc_util import _doc_params

from ._shared_docstrings import doc_entites


@_doc_params(doc_entities=doc_entites)
class DevelopmentalStage(PublicOntology):
    """Developmental Stage.

    1. Developmental Stage Ontology
    https://github.com/obophenotype/developmental-stage-ontologies

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        organism: Literal["human", "mouse"] | None = None,
        source: Literal["hsapdv", "mmusdv"] | None = None,
        version: Literal["2025-01-23", "2024-05-28", "2020-03-10"] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(
            source=source,
            version=version,
            organism=organism,
            include_id_prefixes={"hsapdv": ["HsapDv:"], "mmusdv": ["MmusDv:"]},
            include_rel="part_of",
            **kwargs,
        )
